package com.example.greenwatch;

import android.graphics.ImageFormat;
import android.media.Image;
import android.os.Bundle;
import android.util.Size;

import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.camera.core.CameraSelector;
import androidx.camera.core.ImageAnalysis;
import androidx.camera.lifecycle.ProcessCameraProvider;
// PREVIEW
//import androidx.camera.view.PreviewView;
import androidx.core.content.ContextCompat;

import com.google.common.util.concurrent.ListenableFuture;

import java.nio.ByteBuffer;
import java.util.concurrent.ExecutionException;

public class KameraAktivitaet extends AppCompatActivity {

    // PREVIEW
    // private PreviewView previewView;
    private boolean isMotionDetected;
    private boolean camera_alarm;
    private TextView alarm;
    private ByteBuffer previousBuffer;
    private int previousWidth;
    private int previousHeight;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // NUR FÜR TESTS
        alarm = findViewById(R.id.textView);

        // PREVIEW //
        // previewView = findViewById(R.id.previewView);

        final ListenableFuture<ProcessCameraProvider> cameraProviderFuture = ProcessCameraProvider.getInstance(this);
        cameraProviderFuture.addListener(() -> {
            try {
                bindImageAnalysis(cameraProviderFuture.get());
            } catch (ExecutionException | InterruptedException e) {
                e.printStackTrace();
            }
        }, ContextCompat.getMainExecutor(this));
    }

    private void bindImageAnalysis(ProcessCameraProvider cameraProvider) {
        ImageAnalysis.Builder builder = new ImageAnalysis.Builder();
        builder.setTargetResolution(new Size(640, 480));
        builder.setBackpressureStrategy(ImageAnalysis.STRATEGY_KEEP_ONLY_LATEST);
        ImageAnalysis imageAnalysis = builder.build();
        imageAnalysis.setAnalyzer(
                ContextCompat.getMainExecutor(this),
                imageProxy -> {

                    int imageFormat = imageProxy.getFormat();

                    if (imageFormat == ImageFormat.YUV_420_888) {

                        Image currentImage = imageProxy.getImage();

                        if (previousHeight != 0) {
                            assert currentImage != null;
                            isMotionDetected = compareFrames(currentImage);
                        }

                        assert currentImage != null;
                        Image.Plane[] planes = currentImage.getPlanes();
                        ByteBuffer buffer = planes[0].getBuffer().duplicate();
                        previousBuffer = ByteBuffer.allocateDirect(buffer.remaining());
                        previousBuffer.put(buffer);
                        previousWidth = currentImage.getWidth();
                        previousHeight = currentImage.getHeight();

                        currentImage.close();

                        if (isMotionDetected) {
                            alarm.setText("ALARM");
                            camera_alarm = true;
                        } else {
                            camera_alarm = false;
                            alarm.setText("OK");
                        }

                    }

                    imageProxy.close();
                });

        // Preview preview = new Preview.Builder().build();

        CameraSelector cameraSelector = new CameraSelector.Builder().requireLensFacing(CameraSelector.LENS_FACING_BACK).build();

        // PREVIEW //
        // preview.setSurfaceProvider(previewView.getSurfaceProvider());

        cameraProvider.bindToLifecycle(this, cameraSelector, imageAnalysis);

    }

    // Bildverarbeitung zur Bewegungserkennung
    private boolean compareFrames(Image currentImage) {

        ByteBuffer currentBuffer = currentImage.getPlanes()[0].getBuffer();

        int currentWidth = currentImage.getWidth();
        int currentHeight = currentImage.getHeight();

        if (previousWidth != currentWidth || previousHeight != currentHeight) {
            return false;
        }

        for (int row = 0; row < previousHeight; row++) {
            for (int col = 0; col < previousWidth; col++) {

                int previousIndex = row * previousWidth + col;
                int currentIndex = row * currentWidth + col;

                int previousPixel = previousBuffer.get(previousIndex) & 0xFF;
                int currentPixel = currentBuffer.get(currentIndex) & 0xFF;

                int pixelDifference = Math.abs(previousPixel - currentPixel);
                int threshold = 120;

                if (pixelDifference > threshold) {
                    // Testzwecke
                    //String text = String.valueOf(pixelDifference);
                    return true;
                }
            }
        }
        return false;
    }
}